@@ -4,7 +4,7 @@ from collections.abc import Iterable
 from typing import TYPE_CHECKING, Any, cast, overload
 from typing_extensions import deprecated
 
-from aiohttp import FormData
+from aiohttp import ClientSession, ClientTimeout, FormData
 from graia.amnesia.builtins.aiohttp import AiohttpClientService
 from launart import Launart
 
@@ -20,7 +20,6 @@ from satori.model import (
     LoginPartial,
     Member,
     MessageObject,
-    MessageReceipt,
     Meta,
     Order,
     PageDequeResult,
@@ -39,8 +38,13 @@ if TYPE_CHECKING:
 class ApiProtocol:
     def __init__(self, account: Account):
         self.account = account
+        try:
+            self.session = Launart.current().get_component(AiohttpClientService).session
+        except (LookupError, ValueError):
+            self.session = ClientSession()
+        self.timeout = ClientTimeout(self.account.config.timeout or 300)
 
-    async def download(self, url: str):
+    async def download(self, url: str) -> bytes:
         """访问资源链接。"""
         endpoint = self.account.ensure_url(url)
         aio = Launart.current().get_component(AiohttpClientService)
@@ -67,7 +71,7 @@ class ApiProtocol:
             "Satori-Platform": self.account.platform,
             "Satori-User-ID": self.account.self_id,
         }
-        aio = Launart.current().get_component(AiohttpClientService)
+
         if multipart:
             data = FormData(quote_fields=False)
             if params is None:
@@ -78,68 +82,74 @@ class ApiProtocol:
                     data.add_field(k, v["value"], filename=v.get("filename"), content_type=v["content_type"])
                 else:
                     data.add_field(k, v)
-            async with aio.session.post(
+            async with self.session.post(
                 endpoint,
                 data=data,
                 headers=headers,
+                timeout=self.timeout,
             ) as resp:
                 return await validate_response(resp)
-        async with aio.session.request(
+        async with self.session.request(
             method,
             endpoint,
             json=params or {},
             headers=headers,
+            timeout=self.timeout,
         ) as resp:
             return await validate_response(resp)
 
-    async def send(self, event: Event, message: str | Iterable[str | Element]) -> list[MessageReceipt]:
-        """发送消息。返回一个 `MessageReceipt` 对象构成的数组。
+    async def send(self, event: Event, message: str | Iterable[str | Element]) -> list[MessageObject]:
+        """发送消息。返回一个 `MessageObject` 对象构成的数组。
 
         Args:
             event (Event): 当前事件(上下文)
             message (str | Iterable[str | Element]): 要发送的消息
 
         Returns:
-            list[MessageReceipt]: `MessageReceipt` 对象构成的数组
+            list[MessageObject]: `MessageObject` 对象构成的数组
 
         Raises:
             RuntimeError: 传入的事件缺少 `channel` 对象
         """
         if not event.channel:
             raise RuntimeError("Event cannot be replied to!")
-        return await self.send_message(event.channel.id, message)
+        return await self.send_message(event.channel.id, message, event.referrer)
 
     async def send_message(
-        self, channel: str | Channel, message: str | Iterable[str | Element]
-    ) -> list[MessageReceipt]:
-        """发送消息。返回一个 `MessageReceipt` 对象构成的数组。
+        self, channel: str | Channel, message: str | Iterable[str | Element], referrer: dict[str, Any] | None = None
+    ) -> list[MessageObject]:
+        """发送消息。返回一个 `MessageObject` 对象构成的数组。
 
         Args:
             channel (str | Channel): 要发送的频道 ID
             message (str | Iterable[str | Element]): 要发送的消息
+            referrer (dict[str, Any] | None): 消息来源信息，默认为 None
 
         Returns:
-            list[MessageReceipt]: `MessageReceipt` 对象构成的数组
+            list[MessageObject]: `MessageObject` 对象构成的数组
         """
         channel_id = channel.id if isinstance(channel, Channel) else channel
         msg = message if isinstance(message, str) else "".join(str(i) for i in message)
-        return await self.message_create(channel_id=channel_id, content=msg)
+        return await self.message_create(channel_id=channel_id, content=msg, referrer=referrer)
 
     async def send_private_message(
-        self, user: str | User, message: str | Iterable[str | Element]
-    ) -> list[MessageReceipt]:
-        """发送私聊消息。返回一个 `MessageReceipt` 对象构成的数组。
+        self, user: str | User, message: str | Iterable[str | Element], referrer: dict[str, Any] | None = None
+    ) -> list[MessageObject]:
+        """发送私聊消息。返回一个 `MessageObject` 对象构成的数组。
 
         Args:
             user (str | User): 要发送的用户 ID
             message (str | Iterable[str | Element]): 要发送的消息
+            referrer (dict[str, Any] | None): 消息来源信息，默认为 None
 
         Returns:
-            list[MessageReceipt]: `MessageReceipt` 对象构成的数组
+            list[MessageObject]: `MessageObject` 对象构成的数组
         """
         user_id = user.id if isinstance(user, User) else user
         channel = await self.user_channel_create(user_id=user_id)
-        return await self.message_create(channel_id=channel.id, content="".join(str(i) for i in message))
+        return await self.message_create(
+            channel_id=channel.id, content="".join(str(i) for i in message), referrer=referrer
+        )
 
     async def update_message(
         self, channel: str | Channel, message_id: str, message: str | Iterable[str | Element]
@@ -162,22 +172,25 @@ class ApiProtocol:
             content=msg,
         )
 
-    async def message_create(self, channel_id: str, content: str) -> list[MessageReceipt]:
-        """发送消息。返回一个 `MessageReceipt` 对象构成的数组。
+    async def message_create(
+        self, channel_id: str, content: str, referrer: dict[str, Any] | None = None
+    ) -> list[MessageObject]:
+        """发送消息。返回一个 `MessageObject` 对象构成的数组。
 
         Args:
             channel_id (str): 频道 ID
             content (str): 消息内容
+            referrer (dict[str, Any] | None): 消息来源信息，默认为 None
 
         Returns:
-            list[MessageReceipt]: `MessageReceipt` 对象构成的数组
+            list[MessageObject]: `MessageObject` 对象构成的数组
         """
         res = await self.call_api(
             Api.MESSAGE_CREATE,
-            {"channel_id": channel_id, "content": content},
+            {"channel_id": channel_id, "content": content, "referrer": referrer},
         )
         res = cast("list[dict]", res)
-        return [MessageReceipt.parse(i) for i in res]
+        return [MessageObject.parse(i) for i in res]
 
     async def message_get(self, channel_id: str, message_id: str) -> MessageObject:
         """获取特定消息。返回一个 `MessageObject` 对象。
@@ -635,9 +648,7 @@ class ApiProtocol:
             {"channel_id": channel_id, "message_id": message_id, "emoji": emoji},
         )
 
-    async def reaction_delete(
-        self, channel_id: str, message_id: str, emoji: str, user_id: str | None = None
-    ) -> None:
+    async def reaction_delete(self, channel_id: str, message_id: str, emoji: str, user_id: str | None = None) -> None:
         """从特定消息删除某个用户添加的特定表态。
 
         如果没有传入用户 ID 则表示删除自己的表态。
@@ -788,7 +799,8 @@ class ApiProtocol:
         Returns:
             list[Login]: `Login` 对象构成的数组
         """
-        return (await self.meta_get()).logins
+        res = await self.call_api("admin/login.list")
+        return [LoginPartial.parse(i) for i in res]
 
     async def webhook_create(self, url: str, token: str | None = None):
         """创建 Webhook。"""
